
use strict;
use warnings;

no warnings 'redefine';

use Test::More tests => 24;

use Common;
use Data::Dumper;
use File::Slurp;

my $fname;
my $OUT;
my $num1;
my $num2;
my @base_data;

# test strip_crlf

my %strip_crlf_test = (
	"howdy there\n" => "howdy there",
	"howdy \r\nthere\n" => "howdy there",
	"howdy\r\n there\n" => "howdy there",
);

foreach my $test_key (keys %strip_crlf_test)
{
	my $xstring = Common::strip_crlf ($test_key);
	my $xcompare = $strip_crlf_test {$test_key};

	ok ($xstring eq $xcompare, "strip crlf worked");
}

# test copyHashRef

my $strip_crlf_test_ref = \%strip_crlf_test;
my $copy_strip_crlf_test = Common::copyHashRef ($strip_crlf_test_ref);

is_deeply ($copy_strip_crlf_test, $strip_crlf_test_ref, "copyHashRef worked");

my @raw_sma_data = (
	{
		'Date'  => "2014-06-27 00:00:01",
		'Open'  => "95.568000",
		'High'  => "96.487000",
		'Low'   => "95.314000",
		'Close' => "95.570000",
		'Order' => "Date,Open,High,Low,Close",
	},
	{
		'Date'  => "2014-07-04 00:00:01",
		'Open'  => "95.569000",
		'High'  => "95.973000",
		'Low'   => "94.703000",
		'Close' => "95.105000",
		'Order' => "Date,Open,High,Low,Close",
	},
	{
		'Date'  => "2014-07-11 00:00:01",
		'Open'  => "95.103000",
		'High'  => "95.489000",
		'Low'   => "94.381000",
		'Close' => "94.811000",
		'Order' => "Date,Open,High,Low,Close",
	},
);

# Test outputLine

{
	my $ref_input = {
		'a' => 'a1', 'b' => 'b1',
		'c' => 'c1', 'd' => 'd1',
		'e' => 'e1', 'f' => 'f1',
	};

	my @order = qw(a f b e c notthere); # note d is missing
	my $compare_string = qq~a1,f1,b1,e1,c1,~;

	my $string = Common::outputLine ($ref_input, @order);

	ok ($string eq $compare_string, "outputLine works correctly");
}

# Test outputLinePrepend

{
	my $ref_input = {
		'a' => 'a1', 'b' => 'b1',
		'c' => 'c1', 'd' => 'd1',
		'e' => 'e1', 'f' => 'f1',
	};

	my @order = qw(a f b e c notthere); # note d is missing
	my $compare_string = qq~xyz,a1,f1,b1,e1,c1,~;

	my $string = Common::outputLinePrepend ($ref_input, "xyz", @order);

	ok ($string eq $compare_string, "outputLinePrepend works correctly");
}

# Test getLineOrder

{
	my @raw_dohlc = (
		{
			'Date'  => "2014-06-27 00:00:01",
			'Open'  => "95.568000",
			'High'  => "96.487000",
			'Low'   => "95.314000",
			'Close' => "95.570000",
			'Order' => "Date,Open,High,Low,Close",
		},
		{
			'Date'  => "2014-07-04 00:00:01",
			'Open'  => "95.569000",
			'High'  => "95.973000",
			'Low'   => "94.703000",
			'Close' => "95.105000",
			'Order' => "Open,High,Low,Close,Date",
		},
		{
			'Date'  => "2014-07-11 00:00:01",
			'Open'  => "95.103000",
			'High'  => "95.489000",
			'Low'   => "94.381000",
			'Close' => "94.811000",
			'Order' => "Low,Date,Open,High,Close",
		},
	);

	my @compare_out_order = qw(Date Open High Low Close);
	my $compare_order_string = qq~Date,Open,High,Low,Close~;

	my ($order_string, @out_order) = Common::getLineOrder ("SHOWORDER", @raw_dohlc);

	ok ($compare_order_string eq $order_string, "SHOWORDER - string correct");
	is_deeply (\@compare_out_order, \@out_order, "SHOWORDER - array correct");

	@compare_out_order = qw(Low Date Open High Close);
	$compare_order_string = qq~Low,Date,Open,High,Close~;

	($order_string, @out_order) = Common::getLineOrder ("SHOWORDERLAST", @raw_dohlc);

	ok ($compare_order_string eq $order_string, "SHOWORDERLAST - string correct");
	is_deeply (\@compare_out_order, \@out_order, "SHOWORDERLAST - array correct");

	($order_string, @out_order) = Common::getLineOrder ("NONSENSE", @raw_dohlc);

	ok ($compare_order_string eq $order_string, "NONSENSE - string correct");
	is_deeply (\@compare_out_order, \@out_order, "NONSENSE - array correct");

	($order_string, @out_order) = Common::getLineOrder (undef, @raw_dohlc);

	ok ($compare_order_string eq $order_string, "undef - string correct");
	is_deeply (\@compare_out_order, \@out_order, "undef - array correct");
}

# Test outputArrayToFile

{
	my @raw_dohlc_01 = (
		{
			'Date'  => "2014-06-27 00:00:01",
			'Open'  => "95.568000",
			'High'  => "96.487000",
			'Low'   => "95.314000",
			'Close' => "95.570000",
			'Order' => "Date,Open,High,Low,Close",
		},
		{
			'Date'  => "2014-07-04 00:00:01",
			'Open'  => "95.569000",
			'High'  => "95.973000",
			'Low'   => "94.703000",
			'Close' => "95.105000",
			'Order' => "Open,High,Low,Close,Date",
		},
		{
			'Date'  => "2014-07-11 00:00:01",
			'Open'  => "95.103000",
			'High'  => "95.489000",
			'Low'   => "94.381000",
			'Close' => "94.811000",
			'Order' => "Low,Date,Open,High,Close",
		},
	);

	my @raw_dohlc_02;

	push (@raw_dohlc_02, "SHOWORDER"); 
	push (@raw_dohlc_02, $raw_dohlc_01 [0]);
	push (@raw_dohlc_02, $raw_dohlc_01 [1]);
	push (@raw_dohlc_02, $raw_dohlc_01 [2]);

	my @raw_dohlc_03;

	push (@raw_dohlc_03, "SHOWORDERLAST"); 
	push (@raw_dohlc_03, $raw_dohlc_01 [0]);
	push (@raw_dohlc_03, $raw_dohlc_01 [1]);
	push (@raw_dohlc_03, $raw_dohlc_01 [2]);

	my $fname = "/tmp/t_outputToFile.txt";
	if (-e $fname) { unlink $fname; }

	Common::outputArrayToFile ($fname, @raw_dohlc_01);

	my $buffer;
	my $compare_buffer;

	$buffer = File::Slurp::read_file ($fname);
	$compare_buffer = qq~95.314000,2014-06-27 00:00:01,95.568000,96.487000,95.570000
94.703000,2014-07-04 00:00:01,95.569000,95.973000,95.105000
94.381000,2014-07-11 00:00:01,95.103000,95.489000,94.811000
~;

	ok ($compare_buffer eq $buffer, "outputArrayToFile - no order type is correct");

	if (-e $fname) { unlink $fname; }

	Common::outputArrayToFile ($fname, @raw_dohlc_02);

	$buffer = File::Slurp::read_file ($fname);
	$compare_buffer = qq~Date,Open,High,Low,Close
2014-06-27 00:00:01,95.568000,96.487000,95.314000,95.570000
2014-07-04 00:00:01,95.569000,95.973000,94.703000,95.105000
2014-07-11 00:00:01,95.103000,95.489000,94.381000,94.811000
~;

	ok ($compare_buffer eq $buffer, "outputArrayToFile - SHOWORDER type is correct");

	if (-e $fname) { unlink $fname; }

	Common::outputArrayToFile ($fname, @raw_dohlc_03);

	$buffer = File::Slurp::read_file ($fname);
	$compare_buffer = qq~Low,Date,Open,High,Close
95.314000,2014-06-27 00:00:01,95.568000,96.487000,95.570000
94.703000,2014-07-04 00:00:01,95.569000,95.973000,95.105000
94.381000,2014-07-11 00:00:01,95.103000,95.489000,94.811000
~;

	ok ($compare_buffer eq $buffer, "outputArrayToFile - SHOWORDERLAST type is correct");

	if (-e $fname) { unlink $fname; }

	$buffer = Common::outputArrayToString (@raw_dohlc_03);

	ok ($compare_buffer eq $buffer, "outputArrayToString - SHOWORDERLAST type is correct");
}

# Test getTS

{
	my $xtime = 1411003818;
	my $compareTS = "20140917203018";

	my $TS = Common::getTS ($xtime);
	ok ($TS eq $compareTS, "getTS is correct");
}

# Test ltrim, rtrim, trim

{
	ok (Common::ltrim ("  123 456 4 ") eq "123 456 4 ", "ltrim is correct");
	ok (Common::rtrim ("  123 456 4 ") eq "  123 456 4", "rtrim is correct");
	ok (Common::trim ("  123 456 4 ") eq "123 456 4", "trim is correct");
}

# Test absval

{
	ok (Common::absval (-1) == 1, "absval neg is correct");
	ok (Common::absval (1) == 1, "absval pos is correct");
}

